#include <stdio.h>
#include "pico/stdlib.h"
#include "pico/rand.h"
#include "hardware/irq.h"
#include "hardware/timer.h"

// #define CDCSERBUFSIZE 100

volatile uint16_t LEDRegister[8] = {0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff}; // one for each 128 LED on the board in 8 banks of 16 put a 1 in the correct bit and the LED will light.
volatile uint32_t HostActiveTimeout=0; // counts number of mS since last serial message. if no activity then blinken-random
volatile uint8_t  ColSel, RegisterSelect; // for the LED scan lines and rows in timer interrupt 
volatile uint8_t  BoardAddress; // board address 0..7 read in from DIP switch

bool timer_callback(repeating_timer_t *rt) // Interrupt callback function every 1ms to copy the LEDRegister[] -> light the LEDs
{
	HostActiveTimeout++;
	
  	// Toggle LEDs for selected register bits - High-side driver so 0=LED-ON !
	if(LEDRegister[RegisterSelect]&0x8000) gpio_put(21,0); else  gpio_put(21,1);
	if(LEDRegister[RegisterSelect]&0x4000) gpio_put(20,0); else  gpio_put(20,1);
	if(LEDRegister[RegisterSelect]&0x2000) gpio_put(19,0); else  gpio_put(19,1);
	if(LEDRegister[RegisterSelect]&0x1000) gpio_put(18,0); else  gpio_put(18,1);
	if(LEDRegister[RegisterSelect]&0x0800) gpio_put(17,0); else  gpio_put(17,1);
	if(LEDRegister[RegisterSelect]&0x0400) gpio_put(16,0); else  gpio_put(16,1);
	if(LEDRegister[RegisterSelect]&0x0200) gpio_put(15,0); else  gpio_put(15,1);
	if(LEDRegister[RegisterSelect]&0x0100) gpio_put(14,0); else  gpio_put(14,1);
	if(LEDRegister[RegisterSelect]&0x0080) gpio_put(13,0); else  gpio_put(13,1);
	if(LEDRegister[RegisterSelect]&0x0040) gpio_put(12,0); else  gpio_put(12,1);
	if(LEDRegister[RegisterSelect]&0x0020) gpio_put(11,0); else  gpio_put(11,1);
	if(LEDRegister[RegisterSelect]&0x0010) gpio_put(10,0); else  gpio_put(10,1);
	if(LEDRegister[RegisterSelect]&0x0008) gpio_put(9,0); else  gpio_put(9,1);
	if(LEDRegister[RegisterSelect]&0x0004) gpio_put(5,0); else  gpio_put(5,1);
	if(LEDRegister[RegisterSelect]&0x0002) gpio_put(4,0); else  gpio_put(4,1);
	if(LEDRegister[RegisterSelect]&0x0001) gpio_put(3,0); else  gpio_put(3,1);

    // enable one scan column at a time (Selected reg LED's ON=1)
    if(ColSel==0) gpio_put(22,1); else  gpio_put(22,0);
    if(ColSel==1) gpio_put(23,1); else  gpio_put(23,0);
    if(ColSel==2) gpio_put(24,1); else  gpio_put(24,0);
    if(ColSel==3) gpio_put(25,1); else  gpio_put(25,0);
    if(ColSel==4) gpio_put(26,1); else  gpio_put(26,0);
    if(ColSel==5) gpio_put(27,1); else  gpio_put(27,0);
    if(ColSel==6) gpio_put(28,1); else  gpio_put(28,0);
    if(ColSel==7) gpio_put(29,1); else  gpio_put(29,0);

	// Column select goes 0-7 round-robin
	// Note we can make this between 7 and 15. A bigger number will DIM LED brightness a adding a little more off time. Do not set greater than 15 or will flicker.
    if(ColSel++ > 7) ColSel=0; 

	// Register select goes 0-7 round-robin
	// Keep separate from ColSel in case we want brightness altered.
    if(RegisterSelect++ > 7) RegisterSelect=0;

    return true;
}

int main() 
{
	uint8_t  rn,pin,addr;
    uint16_t rval,cval; // for random generator
//    uint8_t bptr=0;     // for CDC input message string
//    char serbuf[CDCSERBUFSIZE];   // for CDC input message string
    
	// Initialize GPIOs
    for (pin = 0; pin < 30; pin++) 
	{
        if (pin == 0 || pin == 1 || pin == 2) 
		{
            // Set GPIO 0, 1, and 2 as inputs with pull-up resistors
            gpio_init(pin);
            gpio_set_dir(pin, GPIO_IN);
            gpio_pull_up(pin);
        } 
		else 
		{
            // Set remaining GPIOs as outputs
            gpio_init(pin);
            gpio_set_dir(pin, GPIO_OUT);
            gpio_put(pin, 0); // Initialize outputs to LOW
        }
    }

	// read address off DIP switch (1=GND, 0=Vcc so inverted pins!)
	BoardAddress = !gpio_get(0) + (!gpio_get(1)*2) + (!gpio_get(2)*4);

    // Set up a repeating timer to do LED scan/display at 1 kHz (1 ms interval)
    repeating_timer_t timer;
    add_repeating_timer_ms(-1, timer_callback, NULL, &timer);

  //  stdio_init_all(); // required to initialise the USB CDC Serial (ChatGPT thinks!)

    while (true) 
	{
        if(HostActiveTimeout>3000 && HostActiveTimeout % 200 == 10) // do "random" blinkenlights if no serial data
        {
            rn=get_rand_32() % 8;
            rval=get_rand_32() % 40000;
            if (rval>20000)
              LEDRegister[rn]=LEDRegister[rn] << 1;
            else  
              LEDRegister[rn]= LEDRegister[rn] + rval;
        }
        // This next bit does not work yet but it is supposed to read in a serial string and use it to set a 16-bit value in LEDRegisters[]
        /*cval=getchar_timeout_us(0);
        if(cval!=PICO_ERROR_TIMEOUT)
        {
            if(bptr>CDCSERBUFSIZE-1 || HostActiveTimeout>500) bptr=0; // clr serial in buffer of junk / prevent overflow
            serbuf[bptr++]=cval;
            if(cval==13) // if CR then read message string and act on it
            {
                serbuf[bptr]='\0';
                printf(serbuf);
                bptr=0;
                HostActiveTimeout=0;
            }
        }*/
   }
   return 0;
}